import six



def get_unquote():
    if six.PY2:
        from urllib import unquote as unquote
    else:
        from urllib.parse import unquote as unquote
    return unquote

def get_parse():
    if six.PY2:
        from urlparse import urlparse as parse
    else:
        from urllib.parse import urlparse as parse
    return parse

unquote=get_unquote()
parse=get_parse()


# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTUserActivationParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'email': 'str',
        'password': 'str',
        'first_name': 'str',
        'last_name': 'str',
        'default_company_name': 'str',
        'phone_number': 'str',
        'role': 'int'
    }

    attribute_map = {
        'id': 'id',
        'email': 'email',
        'password': 'password',
        'first_name': 'firstName',
        'last_name': 'lastName',
        'default_company_name': 'defaultCompanyName',
        'phone_number': 'phoneNumber',
        'role': 'role'
    }

    def __init__(self, id=None, email=None, password=None, first_name=None, last_name=None, default_company_name=None, phone_number=None, role=None):  # noqa: E501
        """BTUserActivationParams - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._email = None
        self._password = None
        self._first_name = None
        self._last_name = None
        self._default_company_name = None
        self._phone_number = None
        self._role = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if email is not None:
            self.email = email
        if password is not None:
            self.password = password
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if default_company_name is not None:
            self.default_company_name = default_company_name
        if phone_number is not None:
            self.phone_number = phone_number
        if role is not None:
            self.role = role

    @property
    def id(self):
        """Gets the id of this BTUserActivationParams.  # noqa: E501


        :return: The id of this BTUserActivationParams.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this BTUserActivationParams.


        :param id: The id of this BTUserActivationParams.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def email(self):
        """Gets the email of this BTUserActivationParams.  # noqa: E501


        :return: The email of this BTUserActivationParams.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this BTUserActivationParams.


        :param email: The email of this BTUserActivationParams.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def password(self):
        """Gets the password of this BTUserActivationParams.  # noqa: E501


        :return: The password of this BTUserActivationParams.  # noqa: E501
        :rtype: str
        """
        return self._password

    @password.setter
    def password(self, password):
        """Sets the password of this BTUserActivationParams.


        :param password: The password of this BTUserActivationParams.  # noqa: E501
        :type: str
        """

        self._password = password

    @property
    def first_name(self):
        """Gets the first_name of this BTUserActivationParams.  # noqa: E501


        :return: The first_name of this BTUserActivationParams.  # noqa: E501
        :rtype: str
        """
        return self._first_name

    @first_name.setter
    def first_name(self, first_name):
        """Sets the first_name of this BTUserActivationParams.


        :param first_name: The first_name of this BTUserActivationParams.  # noqa: E501
        :type: str
        """

        self._first_name = first_name

    @property
    def last_name(self):
        """Gets the last_name of this BTUserActivationParams.  # noqa: E501


        :return: The last_name of this BTUserActivationParams.  # noqa: E501
        :rtype: str
        """
        return self._last_name

    @last_name.setter
    def last_name(self, last_name):
        """Sets the last_name of this BTUserActivationParams.


        :param last_name: The last_name of this BTUserActivationParams.  # noqa: E501
        :type: str
        """

        self._last_name = last_name

    @property
    def default_company_name(self):
        """Gets the default_company_name of this BTUserActivationParams.  # noqa: E501


        :return: The default_company_name of this BTUserActivationParams.  # noqa: E501
        :rtype: str
        """
        return self._default_company_name

    @default_company_name.setter
    def default_company_name(self, default_company_name):
        """Sets the default_company_name of this BTUserActivationParams.


        :param default_company_name: The default_company_name of this BTUserActivationParams.  # noqa: E501
        :type: str
        """

        self._default_company_name = default_company_name

    @property
    def phone_number(self):
        """Gets the phone_number of this BTUserActivationParams.  # noqa: E501


        :return: The phone_number of this BTUserActivationParams.  # noqa: E501
        :rtype: str
        """
        return self._phone_number

    @phone_number.setter
    def phone_number(self, phone_number):
        """Sets the phone_number of this BTUserActivationParams.


        :param phone_number: The phone_number of this BTUserActivationParams.  # noqa: E501
        :type: str
        """

        self._phone_number = phone_number

    @property
    def role(self):
        """Gets the role of this BTUserActivationParams.  # noqa: E501


        :return: The role of this BTUserActivationParams.  # noqa: E501
        :rtype: int
        """
        return self._role

    @role.setter
    def role(self, role):
        """Sets the role of this BTUserActivationParams.


        :param role: The role of this BTUserActivationParams.  # noqa: E501
        :type: int
        """

        self._role = role

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTUserActivationParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTRevisionRuleParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'description': 'str',
        'script': 'str',
        'company_id': 'str',
        'revision_list': 'list[str]',
        'rule_type': 'int',
        'validation_regex': 'str'
    }

    attribute_map = {
        'name': 'name',
        'description': 'description',
        'script': 'script',
        'company_id': 'companyId',
        'revision_list': 'revisionList',
        'rule_type': 'ruleType',
        'validation_regex': 'validationRegex'
    }

    def __init__(self, name=None, description=None, script=None, company_id=None, revision_list=None, rule_type=None, validation_regex=None):  # noqa: E501
        """BTRevisionRuleParams - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._description = None
        self._script = None
        self._company_id = None
        self._revision_list = None
        self._rule_type = None
        self._validation_regex = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if script is not None:
            self.script = script
        if company_id is not None:
            self.company_id = company_id
        if revision_list is not None:
            self.revision_list = revision_list
        if rule_type is not None:
            self.rule_type = rule_type
        if validation_regex is not None:
            self.validation_regex = validation_regex

    @property
    def name(self):
        """Gets the name of this BTRevisionRuleParams.  # noqa: E501


        :return: The name of this BTRevisionRuleParams.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this BTRevisionRuleParams.


        :param name: The name of this BTRevisionRuleParams.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def description(self):
        """Gets the description of this BTRevisionRuleParams.  # noqa: E501


        :return: The description of this BTRevisionRuleParams.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this BTRevisionRuleParams.


        :param description: The description of this BTRevisionRuleParams.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def script(self):
        """Gets the script of this BTRevisionRuleParams.  # noqa: E501


        :return: The script of this BTRevisionRuleParams.  # noqa: E501
        :rtype: str
        """
        return self._script

    @script.setter
    def script(self, script):
        """Sets the script of this BTRevisionRuleParams.


        :param script: The script of this BTRevisionRuleParams.  # noqa: E501
        :type: str
        """

        self._script = script

    @property
    def company_id(self):
        """Gets the company_id of this BTRevisionRuleParams.  # noqa: E501


        :return: The company_id of this BTRevisionRuleParams.  # noqa: E501
        :rtype: str
        """
        return self._company_id

    @company_id.setter
    def company_id(self, company_id):
        """Sets the company_id of this BTRevisionRuleParams.


        :param company_id: The company_id of this BTRevisionRuleParams.  # noqa: E501
        :type: str
        """

        self._company_id = company_id

    @property
    def revision_list(self):
        """Gets the revision_list of this BTRevisionRuleParams.  # noqa: E501


        :return: The revision_list of this BTRevisionRuleParams.  # noqa: E501
        :rtype: list[str]
        """
        return self._revision_list

    @revision_list.setter
    def revision_list(self, revision_list):
        """Sets the revision_list of this BTRevisionRuleParams.


        :param revision_list: The revision_list of this BTRevisionRuleParams.  # noqa: E501
        :type: list[str]
        """

        self._revision_list = revision_list

    @property
    def rule_type(self):
        """Gets the rule_type of this BTRevisionRuleParams.  # noqa: E501


        :return: The rule_type of this BTRevisionRuleParams.  # noqa: E501
        :rtype: int
        """
        return self._rule_type

    @rule_type.setter
    def rule_type(self, rule_type):
        """Sets the rule_type of this BTRevisionRuleParams.


        :param rule_type: The rule_type of this BTRevisionRuleParams.  # noqa: E501
        :type: int
        """

        self._rule_type = rule_type

    @property
    def validation_regex(self):
        """Gets the validation_regex of this BTRevisionRuleParams.  # noqa: E501


        :return: The validation_regex of this BTRevisionRuleParams.  # noqa: E501
        :rtype: str
        """
        return self._validation_regex

    @validation_regex.setter
    def validation_regex(self, validation_regex):
        """Sets the validation_regex of this BTRevisionRuleParams.


        :param validation_regex: The validation_regex of this BTRevisionRuleParams.  # noqa: E501
        :type: str
        """

        self._validation_regex = validation_regex

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTRevisionRuleParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from onshape_client.configuration import Configuration
import onshape_client
from pathlib import Path
from ruamel.yaml import YAML
import os
import six
from requests_oauthlib import OAuth2Session




class Client:
    """
    Initialize an Onshape client.

    The default is to pull the configuration from a file located in the user's home
    directory. However, configs can also be passed directly as a dictionary.

    Keys can be obtained from https://dev-portal.onshape.com.

    Parameters
    ----------
    conf_file : :obj:`str`, optional
        The path to the location of an onshape_client_config YAML file, where multiple stacks may be declared.
    configuration : :obj:`dict`, optional
        This overrides the conf_file parameter by supplying a dictionary that looks like the following:
        {
            "base_url": "https://cad.onshape.com",
            "access_key": "YOUR_ACCESS_KEY_HERE",
            "secret_key": "YOUR_SECRET_KEY_HERE"
        }
    stack_key : :obj:`str`, optional
        The key of the name for the particular stack to be used. Only applicable when reading configuration from the
        conf_file.
    """
    __instance = None

    @staticmethod
    def get_client():
        """ get an instance of the client class singleton. """
        if Client.__instance == None:
            raise Exception("Please manually instantiate the client.")
        return Client.__instance

    @staticmethod
    def get_configuration_from_keys_file(keys_file, stack_key):
        try:
            yaml = YAML()
            configurations_file = yaml.load(Path(keys_file))
            final_configuration = configurations_file[stack_key if stack_key else configurations_file['default_stack']]
        except KeyError as e:
            raise KeyError(
                "Your creds file is not constructed as expected. The key: {} was expected and now found.".format(e))
        return final_configuration

    def __init__(self, keys_file="~/.onshape_client_config.yaml", configuration=None, stack_key=None):
        if Client.__instance != None:
            raise Exception("This class is a singleton! Please use 'get_client' for all subsequent calls.")
        keys_file = os.path.expanduser(keys_file)
        if configuration:
            final_configuration = configuration
        elif keys_file:
            final_configuration = self.get_configuration_from_keys_file(keys_file, stack_key)
        else:
            raise EnvironmentError("API keys were not properly set.")

        self._set_configuration(final_configuration)
        self._create_apis()

        if self.get_authentication_method() == "oauth":
            self._set_oauth_session()
            self.authorization_url, self.state = self.oauth.authorization_url(
                self.authorization_uri)

        Client.__instance = self

    def get_authentication_method(self):
        # Prefer OAUTH if specified, otherwise try for apikeys
        if self.client_secret and self.client_id:
            return "oauth"
        elif self.configuration.api_key['SECRET_KEY'] and self.configuration.api_key['ACCESS_KEY']:
            return "api_keys"
        else:
            return None

    def do_oauth_flow(self):
        """Do the oauth flow to set the access token"""
        try:
            self._refresh_access_token()
        except BaseException as e:
            if self.oauth_authorization_method == "localhost_server":
                if six.PY3:
                    from onshape_client.oauth.local_server_python_3 import start_server
                else:
                    from onshape_client.oauth.local_server_python_2 import start_server
                start_server(self._fetch_access_token, self.authorization_url)



    def _set_configuration(self, configuration_dictionary):
        configuration = Configuration()

        configuration.api_key['SECRET_KEY'] = self._get_if_present(configuration_dictionary, 'secret_key')
        configuration.api_key['ACCESS_KEY'] = self._get_if_present(configuration_dictionary, 'access_key')

        self.client_id = self._get_if_present(configuration_dictionary, 'client_id')
        self.client_secret = self._get_if_present(configuration_dictionary, 'client_secret')
        configuration.access_token = self._get_if_present(configuration_dictionary, 'access_token')
        self.refresh_token = self._get_if_present(configuration_dictionary, 'refresh_token')
        self.scope = self._get_if_present(configuration_dictionary, 'scope')
        self.redirect_uri = self._get_if_present(configuration_dictionary, 'redirect_uri')
        self.token_uri = self._get_if_present(configuration_dictionary, 'token_uri')
        self.authorization_uri = self._get_if_present(configuration_dictionary, 'authorization_url')
        self.scope = self._get_if_present(configuration_dictionary, 'scope')
        self.oauth_authorization_method = self._get_if_present(configuration_dictionary, 'oauth_authorization_method')

        configuration.host = self._get_if_present(configuration_dictionary, 'base_url')
        self.configuration = configuration
        return

    def _set_oauth_session(self):
        self.oauth = OAuth2Session(self.client_id, redirect_uri=self.redirect_uri,
                                           scope=self.scope)
        return

    def _refresh_access_token(self):
        # So that the oauth library doesn't complain about scope changing
        os.environ["OAUTHLIB_RELAX_TOKEN_SCOPE"] = "True"

        token_response = self.oauth.refresh_token(
            self.token_uri, refresh_token=self.refresh_token, client_id=self.client_id, client_secret=self.client_secret)
        self._set_oauth_creds_from_token_response(token_response)
        return

    def _fetch_access_token(self, authorization_response):
        try:
            token_response = self.oauth.fetch_token(
            self.token_uri,
            authorization_response=authorization_response,
            client_secret=self.client_secret)
            self._set_oauth_creds_from_token_response(token_response)
        except Warning:
            # This is probably a scope warning.
            pass
        return

    def _set_oauth_creds_from_token_response(self, token_response):
        self.configuration.access_token = token_response["access_token"]
        self.refresh_token = token_response["refresh_token"]
        return


    @staticmethod
    def _get_if_present(dictionary, key):
        return dictionary[key] if key in dictionary else ""

    def _create_apis(self):
        api_client = onshape_client.ApiClient(configuration=self.configuration)

        self.accounts_api = onshape_client.api.AccountsApi(api_client)
        self.app_elements_api = onshape_client.api.AppElementsApi(api_client)
        self.assemblies_api = onshape_client.api.AssembliesApi(api_client)
        self.blob_elements_api = onshape_client.api.BlobElementsApi(api_client)
        self.documents_api = onshape_client.api.DocumentsApi(api_client)
        self.elements_api = onshape_client.api.ElementsApi(api_client)
        self.endpoints_api = onshape_client.api.EndpointsApi(api_client)
        self.metadata_api = onshape_client.api.MetadataApi(api_client)
        self.parts_api = onshape_client.api.PartsApi(api_client)
        self.part_studios_api = onshape_client.api.PartStudiosApi(api_client)
        self.translation_api = onshape_client.api.TranslationsApi(api_client)

